import os
import ftplib
import random
import urllib.request
import re
import xbmc
import xbmcaddon
import xbmcvfs
import sqlite3
import time
import zipfile

#
# =========================
#  1) GRUNDEINSTELLUNGEN
# =========================
#

# Aktivierung
ADDON = xbmcaddon.Addon()
ENABLED = ADDON.getSettingBool('enable_sync')
IS_MAIN_SYSTEM = ADDON.getSettingBool('is_main_system')

# FTP
FTP_BASE_PATH = ADDON.getSettingString('ftp_base_path')
FTP_HOST = ADDON.getSettingString('ftp_host')
FTP_USER = ADDON.getSettingString('ftp_user')
FTP_PASS = ADDON.getSettingString('ftp_pass')

# Favouriten
OVERWRITE_STATIC = ADDON.getSettingBool('overwrite_static')
CUSTOM_FOLDER = ADDON.getSettingString('custom_folder')
SPECIFIC_CUSTOM_FOLDER = ADDON.getSettingString('specific_custom_folder')

# Pfade
FTP_PATH = f"/{FTP_BASE_PATH}/auto_fav_sync/{CUSTOM_FOLDER}/favourites.xml"
LOCAL_FAVOURITES = os.path.join(xbmcvfs.translatePath('special://userdata'), 'favourites.xml')
SUPER_FAVOURITES_PATH = os.path.join(xbmcvfs.translatePath('special://userdata'), 'addon_data', 'plugin.program.super.favourites', 'Super Favourites')
ICON_PATH = os.path.join(ADDON.getAddonInfo('path'), 'resources', 'images', 'icon.png')

# Statische Ordner
STATIC_FOLDERS = ADDON.getSettingString('static_folders').split(',')

# Bilder
IMAGE_LIST_URL = ADDON.getSettingString('image_list_url')
LOCAL_IMAGE_PATH = os.path.join(xbmcvfs.translatePath('special://userdata'), 'marvel.jpg')
ADDON_IMAGE_PATH = os.path.join(xbmcvfs.translatePath('special://home/addons/plugin.video.xstream'), 'fanart.jpg')
ENABLE_IMAGE_ROTATION = ADDON.getSettingBool('enable_image_rotation')

# EXTRA SYNC
ENABLE_ADDON_SYNC = ADDON.getSettingBool('addon_sync')
ENABLE_ADDON_STARTUPFILE = ADDON.getSettingBool('startup_file')
ENABLE_ADDON_OPENWIZARDFILE = ADDON.getSettingBool('openwizard_file')

# Mehrsprachigkeit
LANGUAGE = ADDON.getLocalizedString

def show_notification(message_id, duration=5000, **kwargs):
    """
    Zeigt eine Kodi-Notification an.

    Args:
        message_id (int): Die Message-ID für den Lokalisierungstext.
        duration (int): Anzeigedauer in Millisekunden. Standard 5000.
        **kwargs: Optionale Platzhalter für format() im lok. String.

    Returns:
        None
    """
    message = LANGUAGE(message_id).format(**kwargs)
    xbmc.executebuiltin(f'Notification({LANGUAGE(30001)}, {message}, {duration}, {ICON_PATH})')
    time.sleep(duration / 1000)  # Warte, bis die Benachrichtigung abgeschlossen ist

def ftp_upload(local_path, remote_path):
    """
    Lädt eine lokale Datei auf den FTP-Server hoch.

    Args:
        local_path (str): Pfad zur lokalen Quelldatei.
        remote_path (str): Zielpfad auf dem FTP-Server.

    Returns:
        bool: True bei Erfolg, False bei Fehler.
    """
    try:
        xbmc.log(f"Teste FTP-Verbindung zu {FTP_HOST}", xbmc.LOGINFO)
        with ftplib.FTP(FTP_HOST) as ftp:
            ftp.login(FTP_USER, FTP_PASS)
            xbmc.log(f"Erfolgreich bei FTP-Host {FTP_HOST} eingeloggt.", xbmc.LOGINFO)
            xbmc.log(f"Lade Datei hoch: {local_path} -> {remote_path}", xbmc.LOGINFO)
            with open(local_path, 'rb') as file:
                ftp.storbinary(f'STOR {remote_path}', file)
            xbmc.log(f"Datei erfolgreich hochgeladen: {remote_path}", xbmc.LOGINFO)
        return True
    except ftplib.error_perm as e:
        xbmc.log(f"FTP-Berechtigungsfehler: {str(e)}", xbmc.LOGERROR)
    except Exception as e:
        xbmc.log(f"FTP-Upload fehlgeschlagen: {str(e)}", xbmc.LOGERROR)
        return False

def ftp_download(remote_path, local_path):
    """
    Lädt eine Datei vom FTP-Server herunter und speichert sie lokal.

    Args:
        remote_path (str): Pfad auf dem FTP-Server.
        local_path (str): Lokaler Zielpfad.

    Returns:
        bool: True bei Erfolg, False bei Fehler.
    """
    try:
        with ftplib.FTP(FTP_HOST) as ftp:
            ftp.login(FTP_USER, FTP_PASS)
            with open(local_path, 'wb') as file:
                ftp.retrbinary(f'RETR {remote_path}', file.write)
        return True
    except Exception as e:
        xbmc.log(f"FTP download failed: {str(e)}", xbmc.LOGERROR)
        return False

def ftp_folder_exists(folder_path):
    """
    Prüft, ob ein Ordner auf dem FTP-Server existiert.

    Args:
        folder_path (str): Der zu prüfende Ordnerpfad auf dem FTP-Server.

    Returns:
        bool: True, wenn Ordner existiert, sonst False.
    """
    try:
        with ftplib.FTP(FTP_HOST) as ftp:
            ftp.login(FTP_USER, FTP_PASS)
            ftp.cwd(folder_path)
        return True
    except ftplib.error_perm as e:
        if '550' in str(e):
            return False
        else:
            xbmc.log(f"FTP error: {str(e)}", xbmc.LOGERROR)
            return False

def sync_standard_favourites():
    """
    Synchronisiert die Haupt-Favoriten (favourites.xml).

    Returns:
        bool: True bei Erfolg, sonst False.
    """
    if IS_MAIN_SYSTEM:
        return ftp_upload(LOCAL_FAVOURITES, FTP_PATH)
    else:
        return ftp_download(FTP_PATH, LOCAL_FAVOURITES)

def sync_static_favourites():
    """
    Synchronisiert statische Favoritenordner (z.B. Anime, Horror).

    Hinweis: Diese Funktion hat kein True/False-Return.
    Sie startet Upload/Download, gibt aber kein Ergebnis zurück.
    """
    for folder in STATIC_FOLDERS:
        local_static_path = os.path.join(SUPER_FAVOURITES_PATH, folder, 'favourites.xml')
        remote_static_path = f"/{FTP_BASE_PATH}/auto_fav_sync/{CUSTOM_FOLDER}/{folder}/favourites.xml"
        if IS_MAIN_SYSTEM:
            ftp_upload(local_static_path, remote_static_path)
        else:
            ftp_download(remote_static_path, local_static_path)
            if OVERWRITE_STATIC and folder == SPECIFIC_CUSTOM_FOLDER:
                specific_remote_static_path = f"/{FTP_BASE_PATH}/auto_fav_sync/{SPECIFIC_CUSTOM_FOLDER}/favourites.xml"
                ftp_download(specific_remote_static_path, local_static_path)

def clear_texture_cache():
    """
    Löscht den Kodi-Texture-Cache (Textures13.db).

    Returns:
        None
    """
    db_path = xbmcvfs.translatePath('special://database/Textures13.db')
    if os.path.exists(db_path):
        conn = sqlite3.connect(db_path)
        cursor = conn.cursor()
        cursor.execute("DELETE FROM texture")
        cursor.execute("DELETE FROM sizes")
        conn.commit()
        conn.close()
        xbmc.log("Textures cache cleared", xbmc.LOGINFO)

def download_random_image():
    """
    Lädt ein zufälliges Bild aus der IMAGE_LIST_URL, speichert es lokal und zeigt eine Notification.

    Returns:
        bool: True bei Erfolg, False wenn kein Download erfolgte.
    """
    if not IMAGE_LIST_URL or not ENABLE_IMAGE_ROTATION:
        return False

    try:
        with urllib.request.urlopen(IMAGE_LIST_URL) as response:
            content = response.read().decode('utf-8')
            image_urls = re.findall(r'\[img\](.*?)\[/img\]', content)
            if not image_urls:
                xbmc.log("No image URLs found in the list", xbmc.LOGERROR)
                return False
            random_image_url = random.choice(image_urls)
            xbmc.log(f"Selected random image URL: {random_image_url}", xbmc.LOGINFO)
            with urllib.request.urlopen(random_image_url) as img_response:
                img_data = img_response.read()
                with open(LOCAL_IMAGE_PATH, 'wb') as f:
                    f.write(img_data)
                    xbmc.log(f"Saved image to {LOCAL_IMAGE_PATH}", xbmc.LOGINFO)
                with open(ADDON_IMAGE_PATH, 'wb') as f:
                    f.write(img_data)
                    xbmc.log(f"Saved image to {ADDON_IMAGE_PATH}", xbmc.LOGINFO)
        show_notification(30032, 5000)  # Zufallsbild heruntergeladen

        return True
    except urllib.error.HTTPError as e:
        xbmc.log(f"HTTPError: {e.code} - {e.reason}", xbmc.LOGERROR)
        xbmc.log(f"Response: {e.read().decode()}", xbmc.LOGERROR)
    except urllib.error.URLError as e:
        xbmc.log(f"URLError: {e.reason}", xbmc.LOGERROR)
    except Exception as e:
        xbmc.log(f"Failed to download random image: {str(e)}", xbmc.LOGERROR)
    return False

def copy_custom_startup_file():
    """
    Kopiert eine Custom_Startup.xml in den Skin-Ordner (skin.arctic.zephyr.mod/1080i), falls vorhanden.

    Returns:
        None
    """

    if not ENABLE_ADDON_STARTUPFILE:
        return False  # Falls auf 'false' gesetzt, abbrechen

    source_path = os.path.join(xbmcvfs.translatePath('special://home/addons/script.auto.ftp.sync/extras/skin.arctic.zephyr.mod'), 'Custom_Startup.xml')
    destination_folder = xbmcvfs.translatePath('special://home/addons/skin.arctic.zephyr.mod/1080i')
    destination_path = os.path.join(destination_folder, 'Custom_Startup.xml')

    # Überprüfen, ob der Zielordner existiert
    if not os.path.exists(xbmcvfs.translatePath('special://home/addons/skin.arctic.zephyr.mod')):
        xbmc.log("Zielordner für die Custom_Startup.xml existiert nicht. Vorgang wird übersprungen.", xbmc.LOGINFO)
        return

    try:
        # Datei kopieren und überschreiben
        if not os.path.exists(destination_folder):
            os.makedirs(destination_folder)
        xbmcvfs.copy(source_path, destination_path)
        xbmc.log(f"Custom_Startup.xml wurde erfolgreich nach {destination_path} kopiert und überschrieben.", xbmc.LOGINFO)
    except Exception as e:
        xbmc.log(f"Fehler beim Kopieren der Custom_Startup.xml: {str(e)}", xbmc.LOGERROR)

def copy_uservar_file():
    """
    Kopiert die uservar.py aus dem extras-Verzeichnis in das Add-on plugin.program.openwizard,
    falls vorhanden.

    Returns:
        None
    """
    if not ENABLE_ADDON_OPENWIZARDFILE:
        return False  # Falls auf 'false' gesetzt, abbrechen

    # Quellpfad (uservar.py in extras)
    source_path = os.path.join(
        xbmcvfs.translatePath('special://home/addons/script.auto.ftp.sync/extras/plugin.program.openwizard'), 'uservar.py')
    # Zielordner (plugin.program.openwizard)
    destination_folder = xbmcvfs.translatePath('special://home/addons/plugin.program.openwizard')
    # Zieldatei
    destination_path = os.path.join(destination_folder, 'uservar.py')

    # Überprüfen, ob das Ziel-Add-on überhaupt existiert
    if not os.path.exists(destination_folder):
        xbmc.log("Zielordner für uservar.py (plugin.program.openwizard) existiert nicht. Vorgang wird übersprungen.", xbmc.LOGINFO)
        return

    try:
        # uservar.py kopieren und überschreiben
        if not os.path.exists(destination_folder):
            os.makedirs(destination_folder)
        xbmcvfs.copy(source_path, destination_path)
        xbmc.log(f"uservar.py wurde erfolgreich nach {destination_path} kopiert und überschrieben.", xbmc.LOGINFO)
    except Exception as e:
        xbmc.log(f"Fehler beim Kopieren der uservar.py: {str(e)}", xbmc.LOGERROR)

def sync_favourites():
    """
    Startet die Synchronisation der Standard-Favoriten sowie der statischen Ordner.

    Returns:
        None
    """
    if not CUSTOM_FOLDER:
        show_notification(30023, 5000)  # Ein benutzerdefinierter Ordnername ist erforderlich
        return

    if not ftp_folder_exists(f"/{FTP_BASE_PATH}/auto_fav_sync/{CUSTOM_FOLDER}"):
        show_notification(30024, 5000, folder=CUSTOM_FOLDER)  # Benutzerdefinierter Ordner nicht gefunden
        return
    
    # Mach Upload/Download
    result_std = sync_standard_favourites()  # z.B. True/False zurückgeben
    result_stat = sync_static_favourites()   # z.B. True/False

    if result_std or result_stat:
        show_notification(30025, 5000)  # "Favoriten erfolgreich synchronisiert"
    else:
        show_notification(30029, 5000)  # "Fehler bei Favoriten-Sync"

def sync_addon_data():
    """
    Synchronisiert den addon_data-Ordner (lokal -> FTP / FTP -> lokal) mittels einer ZIP-Datei.

    Returns:
        bool: False, wenn die Funktion nicht ausgeführt wurde (z.B. deaktiviert). Sonst kein bestimmter Rückgabewert bei Erfolg.
    """
    if not ENABLE_ADDON_SYNC:
        return False  # Falls auf 'false' gesetzt, abbrechen
    
    xbmc.log("Starte sync_addon_data() mit ZIP-Variante", xbmc.LOGINFO)
    local_base_path = xbmcvfs.translatePath('special://userdata/addon_data')
    local_zip_path = os.path.join(xbmcvfs.translatePath('special://userdata'), 'addon_data.zip')
    remote_zip_path = f"/{FTP_BASE_PATH}/auto_fav_sync/{CUSTOM_FOLDER}/addon_data.zip"

    def create_zip(source_dir, zip_path):
        """
        Erstellt eine ZIP-Datei von einem Quellverzeichnis.

        Args:
            source_dir (str): Pfad zum Quellordner.
            zip_path (str): Zielpfad für das ZIP.

        Returns:
            None
        """
        try:
            xbmc.log(f"Starte die Erstellung der ZIP-Datei: {zip_path}", xbmc.LOGINFO)
            with zipfile.ZipFile(zip_path, 'w', zipfile.ZIP_DEFLATED) as zipf:
                for root, dirs, files in os.walk(source_dir):
                    for file in files:
                        file_path = os.path.join(root, file)
                        arcname = os.path.relpath(file_path, source_dir)
                        zipf.write(file_path, arcname)
                        xbmc.log(f"Datei zur ZIP hinzugefügt: {file_path} -> {arcname}", xbmc.LOGINFO)
            xbmc.log(f"ZIP-Datei erstellt: {zip_path}", xbmc.LOGINFO)
        except Exception as e:
            xbmc.log(f"Fehler beim Erstellen der ZIP-Datei: {str(e)}", xbmc.LOGERROR)

    def extract_zip(zip_path, target_dir):
        """
        Entpackt eine ZIP-Datei in ein Zielverzeichnis.

        Args:
            zip_path (str): Pfad zur ZIP-Datei.
            target_dir (str): Zielverzeichnis für das Entpacken.

        Returns:
            None
        """
        try:
            with zipfile.ZipFile(zip_path, 'r') as zipf:
                zipf.extractall(target_dir)
                xbmc.log(f"ZIP-Datei erfolgreich entpackt: {zip_path} -> {target_dir}", xbmc.LOGINFO)
        except Exception as e:
            xbmc.log(f"Fehler beim Entpacken der ZIP-Datei: {str(e)}", xbmc.LOGERROR)

    if IS_MAIN_SYSTEM:
        # ================
        # Upload-Zweig
        # ================
        xbmc.log("Hauptsystem erkannt. Beginne ZIP-Erstellung.", xbmc.LOGINFO)
        if os.path.exists(local_base_path):
            create_zip(local_base_path, local_zip_path)
            if os.path.exists(local_zip_path):
                xbmc.log(f"ZIP-Datei vorhanden: {local_zip_path}", xbmc.LOGINFO)
                if ftp_upload(local_zip_path, remote_zip_path):
                    xbmc.log(f"ZIP erfolgreich hochgeladen: {remote_zip_path}", xbmc.LOGINFO)
                    os.remove(local_zip_path)
                    show_notification(30021, 5000)  # z.B. "Addon-Daten erfolgreich hochgeladen"
                else:
                    xbmc.log("FTP-Upload fehlgeschlagen.", xbmc.LOGERROR)
                    show_notification(30030, 5000)  # z.B. "Fehler beim Upload"
            else:
                xbmc.log(f"FEHLER: ZIP-Datei wurde nicht erstellt: {local_zip_path}", xbmc.LOGERROR)
        else:
            xbmc.log("Lokaler Ordner 'addon_data' existiert nicht.", xbmc.LOGERROR)

    else:
        # ===================
        # Download-Zweig
        # ===================
        xbmc.log("Kein Hauptsystem. Versuche ZIP herunterzuladen.", xbmc.LOGINFO)
        # 1. ZIP herunterladen
        if ftp_download(remote_zip_path, local_zip_path):
            xbmc.log(f"ZIP-Datei vom Server heruntergeladen: {local_zip_path}", xbmc.LOGINFO)

            # 2. ZIP entpacken ins addon_data-Verzeichnis
            extract_zip(local_zip_path, local_base_path)

            # 3. Lokale ZIP wieder löschen
            if os.path.exists(local_zip_path):
                os.remove(local_zip_path)
                xbmc.log(f"Lokale ZIP-Datei gelöscht: {local_zip_path}", xbmc.LOGINFO)
            show_notification(30031, 5000)  # "Addon-Daten heruntergeladen & entpackt"
        else:
            xbmc.log("ZIP-Download vom FTP fehlgeschlagen.", xbmc.LOGERROR)
            show_notification(30022, 5000)  # "Fehler beim Herunterladen"

#
# =========================
#   Hauptablauf
# =========================
#

if ENABLED:
    xbmc.log("Funktionen werden ausgeführt.", xbmc.LOGINFO)
    sync_addon_data()
    sync_favourites()
    download_random_image()
    copy_custom_startup_file()
    copy_uservar_file()
    # Clear texture cache and refresh UI
    clear_texture_cache()
    xbmc.executebuiltin('ReloadSkin()')
    xbmc.executebuiltin('Container.Refresh()')
